package com.search.lucene.engine.builder.implementations;

import com.search.lucene.engine.abstractions.ISearchEngine;
import com.search.lucene.engine.builder.abstractions.IEngineBuilder;
import com.search.lucene.engine.implementations.LuceneEngine;
import com.search.lucene.indexers.abstractions.IFileIndexer;
import com.search.lucene.indexers.implementations.CSVFileIndexer;
import com.search.lucene.indexers.implementations.PDFFileIndexer;
import com.search.lucene.indexers.implementations.TextFileIndexer;
import com.search.lucene.searchers.abstractions.ISearcher;
import com.search.lucene.searchers.implementations.CSVFileSearcher;
import com.search.lucene.searchers.implementations.PDFFileSearcher;
import com.search.lucene.searchers.implementations.TextFileSearcher;
import com.search.lucene.settings.IndexPaths;

import java.io.IOException;
import java.util.ArrayList;

public class LuceneEngineBuilder implements IEngineBuilder {
    @Override
    public ISearchEngine build() throws IOException {

        ArrayList<IFileIndexer> indexers = new ArrayList<>();
        indexers.add(new CSVFileIndexer(IndexPaths.CSV_INDEX_FOLDER_PATH));
        indexers.add(new PDFFileIndexer(IndexPaths.PDF_INDEX_FOLDER_PATH));
        indexers.add(new TextFileIndexer(IndexPaths.TEXT_INDEX_FOLDER_PATH));

        ArrayList<ISearcher> searchers = new ArrayList<>();
        searchers.add(new CSVFileSearcher(IndexPaths.CSV_INDEX_FOLDER_PATH));
        searchers.add(new PDFFileSearcher(IndexPaths.PDF_INDEX_FOLDER_PATH));
        searchers.add(new TextFileSearcher(IndexPaths.TEXT_INDEX_FOLDER_PATH));


        return new LuceneEngine(indexers,searchers);
    }
}
